# Problemas de Revisão
# 10.3 - página 268
# funções de plotagem de gráfico (Figura 8.3)

# importando bilbiotecas
import matplotlib.pyplot as p
from numpy import log10, sqrt, arange

# declaração das funções
def acidofraco(conc_acido, constante_dissociacao):
    h = sqrt(constante_dissociacao*conc_acido) 
    return h
    
def tampao(conc_acido, conc_sal, constante_dissociacao):    
    h = constante_dissociacao*(conc_acido/conc_sal)
    return h

def hidrolise(conc_sal, constante_dissociacao, prod_ionicoH2O):

    oh = sqrt((prod_ionicoH2O/constante_dissociacao)*conc_sal)
    return oh

def plotagem(x,y,rx,ry, legenda, titulo = 'Meu gráfico'):
    p.clf()                                         # limpando figuras anteriores
    p.plot(x, y, 'b:')                              # plotando o gráfico
    p.xlabel(rx)                                    # legenda do eixo X
    p.ylabel(ry)                                    # legenda do eixo Y
    p.title(titulo)                                 # título
    p.legend(legenda)                               # legenda do gráfico
    p.grid(color='k', linestyle='-', linewidth=0.1) # linhas de grade
    p.show()                                        # mostrar a figura

def curvapH(conc_acido, volume_acido, constante_dissociacao, conc_base):
    '''
    Parameters
    ----------
    conc_acido : float - concentração da amostra de ácido fracco
    volume_acido : float - volume da amostra de ácido fraco
    constante_dissociacao : float - constante de dissociação do ácido fraco
    conc_base : float - concentração molar da base forte usada na titulação

    Returns - lista com os volumes da base (titulante) e pHs calculados
    -------
    None.

    '''
    # declarando variáveis
    Kw = 1e-14                        # produto iônico da água a 25°C
    n_hAc = conc_acido*volume_acido   # quantidade de matéria (mol)
    # dados da base
    v_naoh = arange(0.0,16.5, 0.5) # lista com volumes de NaOH (mL)
    # criação de uma lista vazia para armazenamento dos valores de pH
    pH = []
    
    # repetindo os cálculos para cada ml de NaOH da lista v_naoH
    for i in range(len(v_naoh)):
        n_naoh = m_naoh*v_naoh[i]
        # 1° teste: quando a amostra ainda é pura   
        if n_naoh == 0.0:
            h = acidofraco(m_hAc,Ka)                            #Eq.8.12
            pH.append(-log10(h))                                #Eq.8.13
        # 2° teste: antes de alcançar o ponto de equivalência
        elif n_naoh < n_hAc:
            mt = (n_hAc - n_naoh)/(v_hAc + v_naoh[i])           #Eq.8.15
            ms = n_naoh/(v_hAc + v_naoh[i])                     #Eq.8.16
            h = tampao(mt,ms,Ka)                                #Eq.8.19
            pH.append(-log10(h))
        # 3° teste: no ponto de equivalência
        elif n_naoh == n_hAc:
            ms = n_hAc/(v_hAc + v_naoh[i])                      #Eq.8.16
            oh = hidrolise(ms,Ka,Kw)                            #Eq.8.21
            pH.append(-log10(Kw/oh))                            #Eq.8.22
        # 4° teste: após o ponto de equivalência    
        else:
            mt = (n_naoh - n_hAc)/(v_hAc + v_naoh[i])           #Eq.8.23
            ms = n_hAc/(v_hAc + v_naoh[i])                      #Eq.8.25
            oh1 = hidrolise(ms,Ka,Kw)                           #Eq.8.26
            oh = mt + oh1                                       #Eq.8.27
            pH.append(-log10(Kw/oh))                            #Eq.8.22    

    return [v_naoh, pH]

# exemplo de aplicação
# declarando variáveis
Ka = 1.75e-5                   # constante de dissociação do ácido acético
# dados do ácido
v_hAc = 50.0                   # volume da amostra (mL)
m_hAc = 0.02                   # concentração em quantidade de matéria (mol/L)
m_naoh = 0.1                   # concentração em quantidade de matéria (mol/L)
[v_naoh, pH] = curvapH(m_hAc, v_hAc, Ka, m_naoh)


#---visualizando o comportamento do pH em um gráfico
rotulo_x = 'NaOH (ml)'
rotulo_y = 'pH'
legenda = 'pH'
titulo = 'Curva de Neutralização'
plotagem(v_naoh, pH, rotulo_x, rotulo_y, legenda, titulo)


