# Problemas de Desenvolvimento
# 14.02 - viscosidade dinâmica

# importando bibliotecas
import matplotlib.pyplot as p
from numpy import arange as intervalo

# definindo a função
def fvisc(T, T0, S, mi0):
   
    '''
    Parameters
    ----------
    T : array ou escalar tipo int ou float
        array de temperaturas em K.
    T0 : float
        escalar com temperatura (K) de referência da substância.
    S : float
        constante de Sutherland (K) para aplicação na fórmula.
    mi0 : float
        valor de referênciad da viscosidade à temperatura T0.

    Returns
    -------
    resultado da viscosidade dinâmica na temperatura desejada T (K)


    '''
    a = T0 + S
    b = T + S
    mi = mi0*(a/b)*(T/T0)**(3/2)
    return mi



# intervalo T
T = intervalo(273,450,5)
T2 = intervalo(350,450,5)

# Ar
mi_ar = 1.716e-5
T0_ar = 273.0
S_ar = 111.0

# CO2
mi_co2 = 1.371e-5
T0_co2 = 273.0
S_co2 = 240.0

# N2
mi_n2 = 1.663e10-5
T0_n2 = 273.0
S_n2 = 107.0

# Vapor
mi_vap = 1.120e-5
T0_vap = 350.0
S_vap = 1064.0

# plotando o gráfico
p.clf()                                           # limpando figuras anteriores
p.plot(T, fvisc(T, T0_ar, S_ar, mi_ar), '-k',
       T, fvisc(T, T0_co2, S_co2, mi_co2), '--b',
       T, fvisc(T, T0_n2, S_n2, mi_n2), '-*r',
       T2, fvisc(T2, T0_vap, S_vap, mi_vap), ':g') # plotando o gráfico

p.xlabel('Temperatura (K)')                     # legenda do eixo X
p.ylabel('Viscosidade (N.s/m2)')                # legenda do eixo Y
p.legend(['Ar', 'CO2', 'N2', 'Vapor'])            # legenda do gráfico
p.grid(color='k', linestyle='-', linewidth=0.1) # linhas de grade
p.show()                                        # mostrar a figura

'''
Devido aos pequenos valores, temos três gráficos com um comportamentom muito
próximo de zero. Seria interessante separar os gráficos, colocando-os em
dois gráficos diferentes com suas respectivas escalas. Que tal usar o subplot?
'''
p.figure(2)
p.subplot(1,2,1)
p.plot(T, fvisc(T, T0_co2, S_co2, mi_co2), '--b',
       T, fvisc(T, T0_ar, S_ar, mi_ar), '-k',
       T2, fvisc(T2, T0_vap, S_vap, mi_vap), ':g') # plotando o gráfico

p.xlabel('Temperatura (K)')                     # legenda do eixo X
p.ylabel('Viscosidade (N.s/m2)')                # legenda do eixo Y
p.legend(['CO2','Ar','Vapor'])            # legenda do gráfico
p.grid(color='k', linestyle='-', linewidth=0.1) # linhas de grade
p.show()                                        # mostrar a figura


p.subplot(1,2,2)
p.plot(T, fvisc(T, T0_n2, S_n2, mi_n2), '-*r')  # plotando o gráfico

p.xlabel('Temperatura (K)')                     # legenda do eixo X
p.ylabel('Viscosidade (N.s/m2)')                # legenda do eixo Y
p.legend(['N2'])            # legenda do gráfico
p.grid(color='k', linestyle='-', linewidth=0.1) # linhas de grade
p.tight_layout()
p.show()     