# Problemas de Desenvolvimento
# 15.01 - melhor valor do pH

# importando bibliotecas

from scipy.optimize import minimize
import matplotlib.pyplot as p
from numpy import linspace as intervalo


# função de plotagem
def plotagem(x,y,rx,ry, legenda, mk='k:', titulo = 'Meu gráfico'):
    #p.clf()                                         # limpando figuras anteriores
    p.plot(x, y, mk)                              # plotando o gráfico
    p.xlabel(rx)                                    # legenda do eixo X
    p.ylabel(ry)                                    # legenda do eixo Y
    p.title(titulo)                                 # título
    p.legend(legenda)                               # legenda do gráfico
    p.grid(color='k', linestyle='-', linewidth=0.1) # linhas de grade
    p.show()                                        # mostrar a figura
    
# declaração da função objetivo
def RA(pH,T):
    ra1 = 100*(249.6 + 6.8003*T + 0.066*T*T)
    ra2 = (-0.0214 + 4.12e-3*pH - 1.908e-4*pH*pH)
    return -(ra1*ra2)


# declaração dos limites (variável = [inferior,superior])
pH = [[4.12, 12.7]]


# estimativas iniciais
pHi = 5.0

# temperatura do problema
T = (25.15) # °C

# chamando a função minimize para resolver o problema
solucao = minimize(RA, pHi, bounds = pH, args=T, method='SLSQP', tol=1e-08)

# carregando o valor ótimo do pH
pHo = solucao.x

# imprimindo a solução
print(f'\nPara um pH={float(pHo): .2f}, temos um RA={-RA(float(pHo), T): .3f}.')
print('Veja o asterisco vermelho no gráfico. ESte é o ponto ótimo!')
# plotando o gráfico da função e mostrando o valor ótimo no gráfico
pHs = intervalo(4.12, 12.7,30)
rx='pH'
ry='RA'
lg = ['fRA', 'RA_Otimo x pH']
plotagem(pHs,-RA(pHs,T),rx,ry, lg)
plotagem(float(pHo),-RA(float(pHo),T),rx,ry,lg,mk='*r')

