# Problemas de Desenvolvimento
# 15.06 - equações diferenciais consecutivas

# importando bibliotecas
from numpy import array
import matplotlib.pyplot as p
from scipy.integrate import solve_ivp


# declaração das EDOs
def EDOs(tau, C0):
    # constantes de velocidade
    k1 = 1.0                                   
    k2 = 5.0                                  
    # declaração das variáveis
    CA, CB, CC, CD = C0
    # cálculo das EDOs
    dAdt = -k1*CA*CB - k1*CA*CC              #Ex. 15.09
    dBdt = -k1*CA*CB                         #Ex. 15.10
    dCdt = k1*CA*CB - k2*CA*CC               #Ex. 15.11
    dDdt = k2*CA*CC                          #Ex. 15.12
    
    # retornando os resultados das EDOs
    return array([dAdt, dBdt, dCdt, dDdt])

# Condições iniciais de A, B, C e D
CA0, CB0, CC0, CD0 = 1.0, 0.5, 0.0, 0.0 # [mol/L]

# intervalo do tempo espacial
t = [0, 7]
# resolvendo o sistema ODE, default method='RK45'
sol = solve_ivp(EDOs, t, [CA0, CB0, CC0, CD0], max_step = 0.01)
print(sol)    # impressão do relatório de solução das EDOs
# dados oriundos do solve
t = sol.t                   # valores de t
CA = sol.y[0]               # concentração de A
CB = sol.y[1]               # concentração de B
CC = sol.y[2]               # concentração de C
CD = sol.y[3]               # concentração de D


# plotando os resultados
p.clf()
p.plot(sol.t, CA, 'g-', sol.t, CB, 'b--')
p.plot(sol.t, CC, 'r:', sol.t, CD, 'k-.')
p.xlabel('tempo (h)')
p.ylabel(r'$dCA/dt$', fontsize = 12)
p.legend([r'$u(dC_A/dt)=-k1C_AC_B-k1C_AC_C$', 
          r'$u((dC_B/dt)=-k1C_AC_B$',
          r'$u(dC_C/dt)=k1C_AC_B-k2C_AC_C$',
          r'$u((dC_D/dt)=-k2C_AC_C$'],
          prop={'size': 12})
p.tight_layout()
p.show() 
