# Problemas de Desenvolvimento
# 15.07 - equações diferenciais temperatura do reator

# importando bibliotecas
from numpy import array
from math import exp, log
import matplotlib.pyplot as p
from scipy.integrate import solve_ivp


# declaração das EDOs
def EDOs(t, C0, a, b):
    # constantes de velocidade
    k1 = 0.87                                  
    # declaração das variáveis
    CA, CB, T = C0
    #CB = C0[1]

    # cálculo das EDOs
    dCAdt = -3*k1*(CA**3)*exp(1/T)              #Ex. 15.14
    dCBdt = 3*k1*(CA**3)*exp(1/T)               #Ex. 15.15
    dTdt = a*exp(1/T)*CA - log(T-b)             #Ex. 15.16
    # retornando os resultados das EDOs
    return array([dCAdt,dCBdt,dTdt])

# a  

# Condições iniciais de A, B, C e D
CA0, CB0 = 2.5, 0.0 # [mol/L]
T0 = 33.7  # °C
# declaração de variáveis
a = 12.5
b = 1.55

# intervalo do tempo espacial
t = [0, 1] # h
# resolvendo o sistema ODE, default method='RK45'
sol = solve_ivp(EDOs,t,[CA0, CB0, T0],args=(a,b),method='BDF',max_step = 0.01)
print(sol)    # impressão do relatório de solução das EDOs
# dados oriundos do solve
t = sol.t                   # valores de t
CA = sol.y[0]               # concentração de A
CB = sol.y[1]               # concentração de B
T = sol.y[2]                # variação de T

# plotando os resultados
p.clf()
p.subplot(1,2,1)
p.plot(sol.t, CA, 'g-', sol.t, CB, 'b--')
p.xlabel('tempo (h)')
p.ylabel(r'$dCA/dt-dCB/dt$', fontsize = 11)
p.legend([r'$u(dC_A/dt)=-3k_1C_A^3exp^{(1/T)}$', 
          r'$u((dC_B/dt)=3k_1C_A^3exp^{(1/T)}$'],
          prop={'size': 11})

p.subplot(1,2,2)
p.plot(sol.t, T, 'k-.')
p.xlabel('tempo (h)')
p.ylabel(r'$dT/dt$', fontsize = 11)
p.legend([r'$u(dT/dt)=aexp^{(1/T)}C_a - log(T-b)$'],prop={'size': 10})
p.tight_layout()
p.show() 


# b
'''
perceba que a temperatura do reator continua crescendo mesmo com 2 horas de 
operação. Se você testar um valor de 4 horas, verá que haverá um pico e,
depois, uma queda na temperatura do reator. O que coincide com o fim da
reação.

'''